MODULE SWARM_MOD
!===============================================================================
! MDULO DO MTODO DO PARTICLE SWARM OPTIMIZATION (PSO)
! (KENNEDY E EBERHART, 1995, IN: PROC. IEEE INTERNATIONAL CONFERENCE ON NEURAL NETWORKS, PERTH, AUSTRALIA, PP. 1942-1948)
! SCHWAAB E PINTO, 
! FUNDAMENTOS DE ESTATSTICA E ESTIMAO DE PARMETROS
! EPAPERS, RIO DE JANEIRO,2007
! SEO 5.6.4
  USE GLOBAL_MOD
  USE OBJF_MOD
  USE ZIGGURAT_MOD
!===============================================================================
  CONTAINS
!===============================================================================
  SUBROUTINE SWARM(FLAG_REGCONF,NIT,NPS,PLMIN,PLMAX,FOBJ,GL)
  !IMPLEMENTAO DO METODO PSO
  INTEGER, INTENT(IN) :: FLAG_REGCONF  ! DEFINE SE CRIA ARQUIVO PARA REGIO DE CONFIANA
  INTEGER, INTENT(IN) :: NPS      ! NMERO DE PARTCULAS NO SWARM
  INTEGER, INTENT(IN) :: NIT    ! NMERO DE ITERAES
  REAL(8), INTENT(IN) :: PLMIN(NPAR)  ! LIMITES INFERIOR E SUPEIOR DO INTERVALO DE BUSCA DOS PARMETROS
  REAL(8), INTENT(IN) :: PLMAX(NPAR)  ! LIMITES INFERIOR E SUPEIOR DO INTERVALO DE BUSCA DOS PARMETROS
  REAL(8), INTENT(OUT) :: FOBJ      ! VALOR FINAL DA FUNO OBJETIVO
  INTEGER, INTENT(IN) :: GL !NUMERO DE GRAUS DE LIBERDADE PARA O TESTE DE CHI QUADRADO
  !CALIBRAO DO SWARM SEGUNDO EXEMPLO 5.10
  REAL(8), PARAMETER :: WO = 0.75   ! VALOR INICIAL E FINAL DO FATOR DE INRCIA
  REAL(8), PARAMETER :: WF = 0.75   ! VALOR INICIAL E FINAL DO FATOR DE INRCIA
  REAL(8), PARAMETER :: C1 = 1.5   ! VALOR DAS PONDERAES DA CONTRIBUIO INDIVIDUAL E GLOBAL
  REAL(8), PARAMETER :: C2 = 1.5   ! VALOR DAS PONDERAES DA CONTRIBUIO INDIVIDUAL E GLOBAL
  REAL(8) :: W        ! VALOR DO FATOR DE INRCIA VARIVEL AO LONGO DA BUSCA
  INTEGER :: I
  INTEGER :: J
  INTEGER :: IT    ! CONTADOR DAS ITERAES
  INTEGER :: POS(1)  ! ARRAY1D DE UM ELEMENTO PARA SALVAR O NDICE DA POSIO DO MENOR VALOR DE UM OUTRO ARRAY 1D
  REAL(8) :: P(NPS,NPAR)    ! MATRIZ COM OS VALORES DOS PARMETROS DE CADA PARTCULA
  REAL(8) :: PVEL(NPS,NPAR)  ! MATRIZ COM OS VALORES DAS VELOCIDADES DE CADA PARTCULA
  REAL(8) :: VMAX(NPAR)    ! VETOR COM AS VELOCIDADES MXIMAS EM CADA DIREO
  REAL(8) :: PPT(NPS,NPAR)  ! MATRIZ COM OS VALORES DOS PARMETROS TIMOS DE CADA PARTCULA
  REAL(8) :: POTM(NPAR)    ! VETOR COM OS PARMETROS TIMOS DO CONJUNTO DE PARTCULAS
  REAL(8) :: F(NPS)      ! VETOR COM OS VALORES DAS FUNES OBJETIVO DA ITERAO ATUAL DE CADA PARTCULA
  REAL(8) :: FPT(NPS)    ! VETOR COM OS VALORES TIMOS PARTICULARES DAS FUNES OBJETIVO DE CADA PARTCULA
  REAL(8) :: FOTM !VALORES OTIMOS NA POPULAO DA FUNCAO OBJETIVO AO LONGO DAS ITERAES
  INTEGER :: SAIDA_SWARM_POPULACAO_FNUM
  !-----------------------------------------------------------------------------
  CALL ZIGSET( TIME()**2 ) !INICIALIZAO DO GERADOR DE NMEROS ALEATRIOS
  !SPLASH
  WRITE(*,"(/,'SWARM:')")
  !CABEALHO
  WRITE(*,"(T1,'ITERATION',T20,'OBJF', T30, 'PABS',T70,"//INT_TO_CHAR(NPAR)//"('PARAM(',I0,')',5X))") (I,I=1,NPAR)
  !INICIO
  IF (NPS .GT. 0) THEN
    FOTM = HUGE(1.D0)
    FPT = HUGE(1.D0)
    ! DEFINE A VELOCIDADE MXIMA COMO A QUE TRANSVERSA A REGIAO DE BUSCA INTEIRA EM UM UNICO PASSO
    VMAX = (PLMAX(:) - PLMIN(:))/2.D0
    ! GERANDO SWARM INICIAL E VELOCIDADES INICIAIS
    DO I=1,NPS
      DO J=1,NPAR
        !POSIES ALEATORIAMENTE (UNIFORME) ENTRE OS LIMITES INFERIOR E SUPERIOR
        P(I,J) = PLMIN(J) + UNI()*(PLMAX(J) - PLMIN(J))
        !VELOCIDADES ALEATORIAMENTE (UNIFORME) ENTRE VMAX E -VMAX
        PVEL(I,J) = VMAX(J)*(2.D0*UNI() - 1.D0)
      ENDDO
    ENDDO
      ! 1 PARTCULA PODE TER O VALOR TESTE DO ARQUIVO DE DADOS OPT
        DO J=1,NPAR
          P(1,J) = PARAM(J)
        ENDDO
      !ABRIR ARQUIVO DE REGIAO DE CONFIANA
      IF(FLAG_REGCONF .EQ. 1) THEN
        OPEN (NEWUNIT=SAIDA_SWARM_POPULACAO_FNUM,FILE='OUTPUT/SAIDA_SWARM_POPULACAO.DAT',STATUS='REPLACE',ACTION='WRITE')
      ENDIF
      !INICIO DAS ITERAS
      DO IT = 1,NIT
        DO I=1,NPS
          !PASSA VALORES DE PARAMETOR DA PARTICULA PARA A MEMORIA GLOBAL. TODO: FAZER PASSAGEM POR ARGUMENTO
          PARAM(:) = P(I,:)
          !CALCULA FUNO OBJETIVO PARA TAIS PARMETROS
          ! TODO: FAZER UMA FUNCTION FUNO OBJETIVO
          CALL OBJF(F(I))
        ENDDO
        IF (FLAG_REGCONF .EQ. 1) THEN
          DO I=1,NPS
            !ARQUIVO  QUE VAI SER PARSED, FORMATAO ESTRITA, SEM CABEALHO
            WRITE(SAIDA_SWARM_POPULACAO_FNUM,"("//INT_TO_CHAR(1+1+1+NPAR)//"(A,', '))") &
            LINT_TO_CHAR(IT),LINT_TO_CHAR(I),DSCI_TO_CHAR(F(I)),(DSCI_TO_CHAR(P(I,J)), J=1,NPAR)
          ENDDO
        ENDIF
        !PROCURAR MENOR OBJF DA ITERAO ATUAL
        POS=MINLOC(F)
        ! VERIFICA SE A MENOR OBJF  DESTA ITERAO  MENOR QUE O TIMO GLOBAL ATUAL
        IF (F(POS(1)) .LT. FOTM) THEN
          ! ATUALIZA O VALOR TIMA DA FUNO OBJETIVO
          FOTM=F(POS(1))
          ! ATUALIZA A POSIO TIMA DOS PARAMETROS
          POTM=P(POS(1),:)
          ! ESCREVE O NOVO TIMO GLOBAL NA TELA
          WRITE(*,"("//INT_TO_CHAR(1+1+1)//"(A,', '),"//INT_TO_CHAR(NPAR)//"(A,', ')"//")") &
          INT_TO_CHAR(IT), SCI_TO_CHAR(FOTM), SCI_TO_CHAR(1.D0 - DCHIDF(FOTM,DFLOAT(GL))), (SCI_TO_CHAR(POTM(J)),J=1,NPAR)
        ENDIF
        ! VERIFICA SE A MENOR FOBJ DE CADA PARTICULA DIMINUIU
        FORALL(I=1:NPS,F(I) .LT. FPT(I))
          !ATUALIZA O MINIMO PARTICULAR DE CADA PARTICULA
          FPT(I) = F(I)
          PPT(I,:) = P(I,:)
        ENDFORALL
        ! DEFINE EVOLUO PARA W, EQUAO 5.33
        W = WO + (WF-WO) * DFLOAT(IT-1) / DFLOAT(NIT-1)
        !ATUALIZAO DO SWARM
        DO I=1,NPS
          DO J=1,NPAR
            !ATUALIZAO DA VELOCIDADE, EQUAO 5.31
            !CONTRIBUIO NA DIREO ATERIOR +
            !CONTRIBUIO NA DIREO DO MELHOR INDIVIDUAL +
            !CONTRIBUIO NA DIREO DO MELHOR GLOBAL
            PVEL(I,J) = W*PVEL(I,J) + C1*UNI()*(PPT(I,J) - P(I,J)) + C2*UNI()*(POTM(J) - P(I,J))
            ! CONTROLE DE VELOCIDADE MAXIMA
            IF( ABS(PVEL(I,J)) .GT. VMAX(J) ) THEN
              PVEL(I,J) = SIGN(VMAX(J),PVEL(I,J))
            ENDIF
            ! ATUALIZA POSIO, EQUAO 5.32
            P(I,J) = P(I,J) + PVEL(I,J)
          ENDDO
        ENDDO
        ! VERIFICA SE OS LIMITES DE BUSCA FORAM ULTRAPASSADOS
        ! POSICIONA A PARTIULA NA FRONTEIRA E REDUZ A VELOCIDADE PARA A PROXIMA ITERAO
        ! LIMITE INFERIOR
        FORALL(I=1:NPS, J=1:NPAR, P(I,J) .LT. PLMIN(J))
          P(I,J) = PLMIN(J)
          PVEL(I,J) = - PVEL(I,J)/2.D0
        ENDFORALL
        !LIMITE SUPERIOR
        FORALL(I=1:NPS,J=1:NPAR, P(I,J) .GT. PLMAX(J))
          P(I,J) = PLMAX(J)
          PVEL(I,J) = - PVEL(I,J)/2.D0
        ENDFORALL
      ENDDO
      !FIM DAS ITERAES: 
      !-------------------------------------------------------------------------
      !ENCERRAMENTO
      FOBJ = FOTM
      PARAM = POTM
      IF(FLAG_REGCONF .EQ. 1) THEN
        CLOSE(SAIDA_SWARM_POPULACAO_FNUM)
      ENDIF
    ELSE
      WRITE(*,'(A)') 'NMERO DE PARTCULAS NO SWARM MENOR OU IGUAL A ZERO, SWARM IGNORADO'
    ENDIF
  ENDSUBROUTINE
ENDMODULE
